// Generated by LiveScript 1.5.0
var async, dashify, fs, handlebars, yaml, path, AwsTerraformFileBuilder;
async = require('async');
dashify = require('dashify');
fs = require('fs-extra');
handlebars = require('handlebars');
yaml = require('js-yaml');
path = require('path');
AwsTerraformFileBuilder = (function(){
  AwsTerraformFileBuilder.displayName = 'AwsTerraformFileBuilder';
  var prototype = AwsTerraformFileBuilder.prototype, constructor = AwsTerraformFileBuilder;
  function AwsTerraformFileBuilder(arg$){
    this.appConfig = arg$.appConfig, this.exocomPort = arg$.exocomPort, this.exocomDns = arg$.exocomDns;
    this.productionConfig = this.appConfig.environments.production;
    this.terraformPath = '/usr/src/terraform';
    fs.ensureDirSync(this.terraformPath);
  }
  AwsTerraformFileBuilder.prototype.emptyTerraformDir = function(){
    return fs.emptyDirSync('/usr/src/terraform', function(err){
      switch (false) {
      case !err:
        return process.stdout.write("Could not clear directory '/usr/src/terraform' inside Docker " + err.message);
      }
    });
  };
  AwsTerraformFileBuilder.prototype.generateTerraform = function(){
    this.generateProviderCredentials();
    this._generateMain();
    this._generateVpc();
    this._generateExocomCluster();
    this._generateExocomService();
    this._generateCluster('public');
    this._generateCluster('private');
    this._generateServices('public');
    return this._generateServices('private');
  };
  AwsTerraformFileBuilder.prototype._generateMain = function(){
    var data;
    this._copyTemplateFile('policies');
    data = {
      appName: this.appConfig.name,
      publicClusterSize: this.productionConfig.providers.aws['public-cluster-size'],
      privateClusterSize: this.productionConfig.providers.aws['private-cluster-size'],
      domainName: this.productionConfig.domain
    };
    return this._renderTemplate({
      data: data,
      fileName: 'main.tf'
    });
  };
  AwsTerraformFileBuilder.prototype.generateProviderCredentials = function(){
    var data;
    data = {
      accessKey: process.env.AWS_ACCESS_KEY_ID,
      secretKey: process.env.AWS_SECRET_ACCESS_KEY,
      region: this.productionConfig.providers.aws.region
    };
    return this._renderTemplate({
      data: data,
      fileName: 'provider.tf'
    });
  };
  AwsTerraformFileBuilder.prototype._generateVpc = function(){
    return this._copyTemplateFile('vpc');
  };
  AwsTerraformFileBuilder.prototype._generateExocomCluster = function(){
    return this._copyTemplateFile('exocom');
  };
  AwsTerraformFileBuilder.prototype._generateExocomService = function(){
    var data;
    this._buildExocomContainerDefinition();
    data = {
      exocomDns: this.exocomDns
    };
    return this._appendToMainScript({
      data: data,
      templateName: 'exocom-service.tf'
    });
  };
  AwsTerraformFileBuilder.prototype._generateCluster = function(type){
    return this._copyTemplateFile(type + "-services");
  };
  AwsTerraformFileBuilder.prototype._generateServices = function(type){
    var serviceName, ref$, serviceData, serviceConfig, data, results$ = [];
    for (serviceName in ref$ = this.appConfig.services[type + ""]) {
      serviceData = ref$[serviceName];
      serviceConfig = yaml.safeLoad(fs.readFileSync(path.join('/var/app', serviceData.location, 'service.yml'), 'utf8'));
      this._buildServiceContainerDefinition(serviceName, this._getImageName(serviceData), serviceConfig);
      data = {
        name: serviceName,
        publicPort: serviceConfig.production.aws['public-port'],
        publicUrl: serviceConfig.production.url
      };
      results$.push(this._appendToMainScript({
        data: data,
        templateName: type + "-service.tf"
      }));
    }
    return results$;
  };
  AwsTerraformFileBuilder.prototype._renderTemplate = function(arg$){
    var data, fileName, srcPath, src, renderedFile;
    data = arg$.data, fileName = arg$.fileName;
    srcPath = path.join(__dirname, "../../templates/aws-terraform/" + fileName);
    src = fs.readFileSync(srcPath, 'utf-8');
    renderedFile = handlebars.compile(src)(data);
    return fs.writeFileSync(this.terraformPath + "/" + fileName, renderedFile);
  };
  AwsTerraformFileBuilder.prototype._appendToMainScript = function(arg$){
    var data, templateName, srcPath, src, renderedText;
    data = arg$.data, templateName = arg$.templateName;
    srcPath = path.join(__dirname, "../../templates/aws-terraform/" + templateName);
    src = fs.readFileSync(srcPath, 'utf-8');
    renderedText = handlebars.compile(src)(data);
    return fs.appendFileSync(this.terraformPath + "/main.tf", renderedText);
  };
  AwsTerraformFileBuilder.prototype._copyTemplateFile = function(fileName){
    var srcPath;
    srcPath = path.join(__dirname, "../../templates/aws-terraform/" + fileName);
    return fs.copySync(srcPath, this.terraformPath + "/" + fileName);
  };
  AwsTerraformFileBuilder.prototype._buildServiceContainerDefinition = function(serviceName, imageName, serviceConfig){
    var containerDefinition, targetPath, this$ = this;
    containerDefinition = [{
      name: "exosphere-" + serviceName + "-service",
      image: imageName,
      cpu: serviceConfig.production.aws.cpu,
      memory: serviceConfig.production.aws.memory,
      command: function(it){
        return it.split(' ');
      }(
      serviceConfig.startup.command),
      portMappings: this._buildPortMappings(serviceConfig),
      environment: this._buildServiceEnvironmentVariables(serviceName, serviceConfig)
    }];
    targetPath = path.join(this.terraformPath, serviceName + "-container-definition.json");
    return fs.writeFileSync(targetPath, JSON.stringify(containerDefinition, null, 2));
  };
  AwsTerraformFileBuilder.prototype._buildPortMappings = function(serviceConfig){
    var portMappings;
    portMappings = [{
      hostPort: 0,
      containerPort: this.exocomPort,
      protocol: 'tcp'
    }];
    if (serviceConfig.production.aws['public-port']) {
      portMappings.push({
        hostPort: 80,
        containerPort: serviceConfig.production.aws['public-port'],
        protocol: 'tcp'
      });
    }
    return portMappings;
  };
  AwsTerraformFileBuilder.prototype._buildServiceEnvironmentVariables = function(serviceName, serviceConfig){
    var environment, dependency;
    environment = [
      {
        name: 'EXOCOM_HOST',
        value: this.exocomDns
      }, {
        name: 'EXOCOM_PORT',
        value: this.exocomPort + ""
      }, {
        name: 'SERVICE_NAME',
        value: serviceName
      }
    ];
    for (dependency in serviceConfig.dependencies) {
      switch (dependency) {
      case 'mongo':
        process.env.MONGODB_USER == null && (fn$());
        process.env.MONGODB_PW == null && (fn1$());
        environment = environment.concat({
          name: 'MONGODB_USER',
          value: process.env.MONGODB_USER
        });
        environment = environment.concat({
          name: 'MONGODB_PW',
          value: process.env.MONGODB_PW
        });
      }
    }
    return environment;
    function fn$(){
      throw new Error("MONGODB_USER not provided");
    }
    function fn1$(){
      throw new Error("MONGODB_PW not provided");
    }
  };
  AwsTerraformFileBuilder.prototype._buildExocomContainerDefinition = function(){
    var containerDefinition, targetPath;
    containerDefinition = [{
      name: 'exocom',
      image: 'originate/exocom',
      cpu: 100,
      memory: 500,
      essential: true,
      command: ['bin/exocom'],
      portMappings: [{
        hostPort: this.exocomPort,
        containerPort: this.exocomPort,
        protocol: 'tcp'
      }],
      environment: [{
        name: 'SERVICE_MESSAGES',
        value: JSON.stringify(
        this._compileServiceMessages())
      }]
    }];
    targetPath = path.join(this.terraformPath, 'exocom-container-definition.json');
    return fs.writeFileSync(targetPath, JSON.stringify(containerDefinition, null, 2));
  };
  AwsTerraformFileBuilder.prototype._getImageName = function(serviceData){
    var serviceConfig;
    serviceConfig = yaml.safeLoad(fs.readFileSync(path.join('/var/app', serviceData.location, 'service.yml'), 'utf8'));
    return serviceConfig.author + "/" + dashify(serviceConfig.title);
  };
  AwsTerraformFileBuilder.prototype._compileServiceMessages = function(){
    var serviceMessages, type, serviceName, ref$, serviceData, serviceConfig;
    serviceMessages = [];
    for (type in this.appConfig.services) {
      for (serviceName in ref$ = this.appConfig.services[type + ""]) {
        serviceData = ref$[serviceName];
        serviceConfig = yaml.safeLoad(fs.readFileSync(path.join('/var/app', serviceData.location, 'service.yml'), 'utf8'));
        serviceMessages.push({
          name: serviceName,
          receives: serviceConfig.messages.receives,
          sends: serviceConfig.messages.sends,
          namespace: serviceData.namespace
        });
      }
    }
    return serviceMessages;
  };
  return AwsTerraformFileBuilder;
}());
module.exports = AwsTerraformFileBuilder;